unit TextEditor3U;

interface

uses
  SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  QTypes, QMenus, QStdCtrls, QComCtrls, QImgList, QExtCtrls, QActnList,
  QStdActns;

type
  TfrmTextEditor = class(TForm)
    mnuMainMenu: TMainMenu;
    memText: TMemo;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    mniFile: TMenuItem;
    mniSeparator: TMenuItem;
    mniEdit: TMenuItem;
    mniHelp: TMenuItem;
    mniOptions: TMenuItem;
    imlImages: TImageList;
    sbInfo: TStatusBar;
    timClock: TTimer;
    tbMainToolBar: TToolBar;
    alActions: TActionList;
    actNew: TAction;
    actOpen: TAction;
    actSave: TAction;
    actExit: TAction;
    actWordWrap: TAction;
    actAbout: TAction;
    EditCut1: TEditCut;
    EditCopy1: TEditCopy;
    EditPaste1: TEditPaste;
    EditSelectAll1: TEditSelectAll;
    EditDelete1: TEditDelete;
    New1: TMenuItem;
    Open1: TMenuItem;
    actSave1: TMenuItem;
    Exit1: TMenuItem;
    WordWrap1: TMenuItem;
    About1: TMenuItem;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    Delete1: TMenuItem;
    N1: TMenuItem;
    SelectAll1: TMenuItem;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure memTextChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure timClockTimer(Sender: TObject);
    procedure actNewExecute(Sender: TObject);
    procedure actOpenExecute(Sender: TObject);
    procedure actSaveExecute(Sender: TObject);
    procedure actSaveUpdate(Sender: TObject);
    procedure actExitExecute(Sender: TObject);
    procedure actWordWrapExecute(Sender: TObject);
    procedure actAboutExecute(Sender: TObject);
  private
    { Private declarations }
    FModified: Boolean;
    FFileName: WideString;
    procedure SetFileName(const FileName: WideString);
  public
    { Public declarations }
  end;

var
  frmTextEditor: TfrmTextEditor;

implementation

{$R *.xfm}

procedure TfrmTextEditor.SetFileName(const FileName: WideString);
begin
  FFileName := FileName;
  if Length(FFileName) = 0 then
    Caption := '<New File>'
  else
    Caption := FFileName
end;

procedure TfrmTextEditor.FormCreate(Sender: TObject);
begin
  //Get the form to say we have a new file to start with
  SetFileName('')
end;

procedure TfrmTextEditor.memTextChange(Sender: TObject);
begin
  FModified := True
end;

procedure TfrmTextEditor.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  actNew.Execute;
  if FModified then
    CanClose := False
end;

procedure TfrmTextEditor.timClockTimer(Sender: TObject);
begin
  sbInfo.Panels[1].Text := FormatDateTime('h:nn:ss am/pm "on" ddd, d mmm, yyy', Now)
end;

procedure TfrmTextEditor.actNewExecute(Sender: TObject);
begin
  if FModified then
    case MessageDlg('Save changes?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes:    actSave.Execute;
      mrNo:     FModified := False;
      mrCancel: Exit;
    end;
  if not FModified then
  begin
    SetFileName('');
    memText.Clear;
    FModified := False
  end
end;

procedure TfrmTextEditor.actOpenExecute(Sender: TObject);
begin
  if dlgOpen.Execute then
  begin
    actNew.Execute;
    if not FModified then
    begin
      memText.Lines.LoadFromFile(dlgOpen.FileName);
      SetFileName(dlgOpen.FileName);
      FModified := False
    end
  end
end;

procedure TfrmTextEditor.actSaveExecute(Sender: TObject);
begin
  if Length(FFileName) = 0 then
    if dlgSave.Execute then
      SetFileName(dlgSave.FileName);
  if Length(FFileName) > 0 then
  begin
    memText.Lines.SaveToFile(FFileName);
    FModified := False
  end
end;

procedure TfrmTextEditor.actSaveUpdate(Sender: TObject);
begin
  actSave.Enabled := FModified
end;

procedure TfrmTextEditor.actExitExecute(Sender: TObject);
begin
  Close
end;

procedure TfrmTextEditor.actWordWrapExecute(Sender: TObject);
begin
  memText.WordWrap := not actWordWrap.Checked;
  actWordWrap.Checked := memText.WordWrap
end;

procedure TfrmTextEditor.actAboutExecute(Sender: TObject);
begin
  ShowMessage('Text Editor Application')
end;

end.
